# toAPK

## What?
Conversion of Arch PKGBUILDs, Gentoo Ebuilds, and VOID Templates to Alpine APKBUILD
Written in Fennel Lisp!

## Overview:
![toAPK Useage Overview](img/overview.gif)

### Road map:
Long term I'd like to see the following for toAPK

- [x] PKGBUILD -> APKBUILD
- [x] Void Template -> APKBUILD
- [x] Ebuild -> APKBUILD
- [ ] Deb -> APKBUILD
- [ ] APKBUILD -> PKGBUILD
- [ ] Custom function mapping
- [x] Custom variable mapping
- [x] Url fetch w/ package type handling
- [ ] Creation of APKBUILDs from git urls
- [x] toAPK static compilation
- [x] Packaged for Alpine Linux

If you'd like to contribute, please feel free to make a MR, or contact me!

## Building:
You can build toAPK by calling the following:

### Static Fennel Binary
* make compile-bin
* make install-bin

### Executable Lua Script
* make compile-lua
* make install-lua

## Usage:
```
;;===== toAPK v0.9 =====
Author: Will Sinatra <wpsinatra@gmail.com>
License: GPLv3

;;===== Usage =====
toAPK -a ARCH_PKGBUILD  | Parse Arch Linux PKGBUILD
toAPK -v VOID_TEMPLATE  | Parse VOID Linux Template
toAPK -e GENTOO_EBUILD  | Parse Gentoo Linux Ebuild
toAPK -u $URL           | Pull URL & print to STDOUT
toAPK -u -a/v/e $NAME   | Pull distro specific package by name
toAPK -c                | Configure .toAPK
toAPK -h                | Helpful help is helpful
```

Handling various different build system conversions is no small task, toAPK has the following issues. All of the below said, you will likely only need to focus on modifying the build/install functions to be APKBUILD compliant. Eventually handling logic for that will be added.

### PKGBUILDs
Some Arch specific logic is not translated, such as builddirs
Custom variables are not carried over
Conversion is good enough to produce a nearly no work necessary APKBUILD

### Gentoo
Custom variables are not carried over
Package parsing is currently unrefined, it's a middling start on an APKBUILD

### VOID
Custom variables are not carried over
Void vfunctions are semi translated (ie: vman translates into install -Dm644, but needs additional work to be useful)
Conversion is good enough to produce a middling work APKBUILD

## Example:
Using the PKGBUILD from the AUR for fennel, toAPK produces the following output

```
[wsinatra@domus toAPK]$ toAPK -a PKGBUILD 
# Contributor: Will Sinatra <wpsinatra@gmail.com>
# Maintainer: Will Sinatra <wpsinatra@gmail.com>
pkgname=fennel
pkgver=0.3.2
pkgrel=0
pkgdesc="A Lua Lisp language"
url="https://github.com/bakpakin/Fennel"
arch="x86_64"
license="MIT"
depends="lua"
source="fennel-$pkgver.tar.gz::https://github.com/bakpakin/Fennel/archive/$pkgver.tar.gz"

package() {


 install -Dm644 "fennel.lua" "fennelview.fnl" \
  -t "$pkgdir/usr/lib/lua/${_lua_version}"
 
 install -Dm755 "fennel" \
  "$pkgdir/usr/bin/fennel"
}
```

This differs slightly from the APKBUILD found in Alpine's repo that I authored by hand, however it's only slightly removed. A check function needs to be added, there is superfluous white space, but otherwise the output is a compliant APKBUILD. The same can be done with Void Linux Template files, though not everything has been accounted for at this time.

## Configure:
toAPK stores user configuration in ~/.toAPK. If this configuration is missing package conversion won't function.

The .toAPK file can be generated by calling toAPK -c, this will drop you into a simple input prompt.
```
devius:~/Development/toAPK$ toAPK -c
Generating .toAPK..
Enter your first and last name.
Will Sinatra
Enter your email.
wpsinatra@gmail.com
.toAPK created at /home/wsinatra/.toAPK
```

alternately the file can be created by hand following the format below
```
flastname:Will Sinatra
email:<wpsinatra@gmail.com>
```

## Tests:
make test-all:
- Compiles toAPK
- Converts a suite of Arch, Gentoo, and Void packages pulled from their respective repos.

## License:
GNU General Public License v3.0